using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Implicit2D;

namespace Implicit2DTest
{
    public partial class Form1 : Form
    {
        List<ImplicitPoint> m_vPoints;
        ImplicitBlend m_blend;
        ImplicitPoint m_selected;

        bool m_bPolylineValid;
        MarchingQuads m_march;
        int m_nSubdivisions;
        Box2f m_bounds;
        float m_fIsoValue;
        
        public Form1() {
            InitializeComponent();

            m_bounds = new Box2f(canvas.Width, canvas.Height);
            m_nSubdivisions = 50;
            m_fIsoValue = 0.5f;
            m_march = new MarchingQuads(m_nSubdivisions, m_bounds, m_fIsoValue);
            canvas_Resize(null,null);

            m_blend = new ImplicitBlend();
            m_vPoints = new List<ImplicitPoint>();

            ImplicitPoint newPoint1 = new ImplicitPoint(200,200,150);
            m_vPoints.Add( newPoint1 );
            ImplicitPoint newPoint2 = new ImplicitPoint(300,300,150);
            m_vPoints.Add( newPoint2 );
            m_blend.AddChild(newPoint1);
            m_blend.AddChild(newPoint2);

            m_selected = null;

            UpdatePolyLine();
            canvas_Paint(null,null);
        }


        public void UpdatePolyLine() {
            if ( m_bPolylineValid )
                return;

            //m_march.ClearStroke();
            //m_march.ClearSeedPoints();
            m_march = new MarchingQuads(m_nSubdivisions, m_bounds, m_fIsoValue);
            foreach ( ImplicitPoint pt in m_vPoints )
                m_march.AddSeedPoint( pt.x, pt.y );
            m_march.Polygonize( m_blend );
            m_bPolylineValid = true;
        }
        
        void InvalidatePolyline() {
            m_bPolylineValid = false;
        }

        public void DrawPolyLine(Graphics g, Pen p) {
            UpdatePolyLine();

            PolyLine line = m_march.Stroke;
            foreach ( PolyLine.Edge e in line.Edges ) {
                PolyLine.Vertex v1 = line.GetVertex(e.v1);
                PolyLine.Vertex v2 = line.GetVertex(e.v2);
                g.DrawLine(p, v1.x, v1.y, v2.x, v2.y);
            }
        }

        private void canvas_Paint(object sender, PaintEventArgs e) {
            Graphics g = Graphics.FromImage(canvas.Image);
            g.Clear(Color.White);

            Brush b = Brushes.Red;
            foreach ( ImplicitPoint pt in m_vPoints ) 
                g.FillEllipse( b, pt.x - 5, pt.y - 5, 10, 10 );

            Pen p = new Pen(Color.Black, 3.0f);
            DrawPolyLine(g, p);
        }

        private void canvas_Resize(object sender, EventArgs e) {
            if ( canvas.Width > 0 ) {
                canvas.Image = new Bitmap( canvas.Width, canvas.Height );
                Graphics g = Graphics.FromImage(canvas.Image);
                g.Clear(Color.White);
                m_bounds = new Box2f(canvas.Width, canvas.Height);
                m_march.Bounds = m_bounds;
            }
        }

        private void canvas_MouseDown(object sender, MouseEventArgs e) {
            Vector2f ptClick = new Vector2f(e.X, e.Y);
            if ( e.Button == MouseButtons.Left ) {
                m_selected = null;
                int nNearest = -1;
                float fNearest = float.MaxValue;
                for ( int i = 0; i < m_vPoints.Count; ++i ) {
                    float fDist = m_vPoints[i].Center.Dist(ptClick);
                    if ( fDist < fNearest ) {
                        nNearest = i;
                        fNearest = fDist;
                    }
                }
                if ( nNearest != -1 && fNearest < 10 ) {
                    m_selected = m_vPoints[nNearest];
                    canvas.Invalidate();
                }

            } else if ( e.Button == MouseButtons.Right ) {
                ImplicitPoint pt = new ImplicitPoint(ptClick.X, ptClick.Y, 100);
                m_vPoints.Add(pt);
                m_blend.AddChild(pt);
                InvalidatePolyline();
                canvas.Invalidate();
            }
        }

        private void canvas_MouseMove(object sender, MouseEventArgs e) {
            if ( m_selected != null ) {
                m_selected.Center = new Vector2f(e.X, e.Y);
                InvalidatePolyline();
                canvas.Invalidate();
            }
        }

        private void canvas_MouseUp(object sender, MouseEventArgs e) {
            m_selected = null;
            canvas.Invalidate();
        }


        private void GridResTrackbar_Scroll(object sender, EventArgs e) {
            m_nSubdivisions = GridResTrackbar.Value;
            InvalidatePolyline();
            canvas.Invalidate();         
        }

        private void IsoValueTrackbar_Scroll(object sender, EventArgs e) {
            m_fIsoValue = (float)(IsoValueTrackbar.Value) / 100.0f;
            InvalidatePolyline();
            canvas.Invalidate();         
        }


    }
}